public abstract class Geom {
        String type;
        String nom;
        Geom liaison;
        
        public static double sqr(double x) {
            return x*x;
        }
        
        
}

public class GPoint extends Geom {
		double x,y;
		
		GPoint(double x,double y) {
			this.x=x;
			this.y=y;
		}
		
		GPoint(GDroite d1, GDroite d2) {
		    double delta = d1.v * d2.u - d1.u * d2.v;
		    x = (d1.h * d2.v - d1.v * d2.h)/delta;
		    y = (d1.h * d2.u - d1.u * d2.h)/delta;
		}
		GPoint(GDroite d, double t) {
		     x = -d.h*d.u-d.v*t;
		     y = -d.h*d.v+d.u*t;
		}
		GPoint(GCercle c, double t) {
		     x = c.x + c.r * Math.cos(t);
		     y = c.y + c.r * Math.sin(t);
		}
		
		public GDroite droite(GPoint m) {
		     return new GDroite(this, m);
		}
		
		public GPoint milieu(GPoint m) {
		     return new GPoint((x+m.x)/2,(y+m.y)/2);
		}
		
		public GDroite perpendiculaire(GDroite d) {
		     return new GDroite(-d.v,d.u,d.v*x-d.u*y);
		}
		
		public GDroite parallele(GDroite d) {
		     return new GDroite(d.u,d.v,-d.u*x-d.v*y);
		}
		
		public GDroite mediatrice(GPoint m) {
		     return new GDroite(x-m.x,y-m.y,(-Geom.sqr(x)+Geom.sqr(m.x)-Geom.sqr(y)+Geom.sqr(m.y))/2);
		}
		
		public GPoint projection(GDroite d) {
		     double t = d.u*x+d.v*y+d.h;
		     return new GPoint(x-t*d.u,y-t*d.v);
		}
		
		public Geom symetrie(GPoint m) {
		     return new GPoint(2*m.x-x,2*m.y-y);
		}
		public Geom symetrie(GDroite d) {
		     double t = d.u*x+d.v*y+d.h;
		     return new GPoint(x-2*t*d.u,y-2*t*d.v);
		}
		public Geom symetrie(GCercle c) {
		     double lambda = c.r/(Geom.sqr(x-c.x)+Geom.sqr(y-c.y));
		     return new GPoint(c.x+lambda*(x-c.x), c.y+lambda*(y-c.y));
		}
		
		public double distance(GPoint m) {
		     return Math.sqrt(Geom.sqr(x-m.x)+Geom.sqr(y-m.y));
		}
		
		public double distance(GDroite d) {
		    return Math.abs(d.u*x+d.v*y+d.h);
		}
		
		public boolean appartient(GDroite d, double tolerance) {
		    return (Math.abs(d.u*x+d.v*y+d.h) < tolerance);
		}
		
		public boolean appartient(GDroite d) {
		    return appartient(d,1e-6);
		}
		public boolean appartient(GCercle c, double tolerance) {
		    return (Math.abs(Geom.sqr(c.x-x)+Geom.sqr(c.y-y)-c.r*c.r) < tolerance);
		}
		
		public boolean appartient(GCercle c) {
		    return appartient(c,1e-6);
		}
		
		public static boolean alignes(GPoint m1, GPoint m2, GPoint m3) {
		    return m1.appartient(new GDroite(m2,m3));
		}
}

public class GDroite extends Geom {
        double u,v,h;
        
        GDroite(double u, double v, double h){
            double d = Math.sqrt(u*u+v*v);
            this.u = u/d;
            this.v = v/d;
            this.h = h/d;
            
        }
        
        GDroite(GPoint m1, GPoint m2) {
           this(m2.y-m1.y,m1.x-m2.x,m1.y*m2.x-m1.x*m2.y);
        }
        
        public GPoint intersection(GDroite d) {
             return new GPoint(this,d);
        }
        
        public GPoint[] intersections(GCercle c) {
             GPoint[] tableau = new GPoint[2];
             double delta = Math.sqrt(Geom.sqr(c.x)-Geom.sqr(u*c.x+v*c.y+h));
             double b1 = -c.y*u*v+c.x*v*v-u*h;
             double b2 = -c.x*u*v+c.y*u*u-v*h;
             tableau[0] = new GPoint(b1+v*delta,b2-u*delta);
             tableau[1] = new GPoint(b1-v*delta,b2+u*delta);
             return tableau;
        }
        
        public GDroite perpendiculaire(GPoint m) {
		     return new GDroite(-v,u,v*m.x-u*m.y);
		}
		
		public GDroite parallle(GPoint m) {
		     return new GDroite(u,v,u*m.x-v*m.y);
		}
		
		public GDroite[] bissectrices(GDroite d) {
		     GDroite[] tableau = new GDroite[2];
		     tableau[0] = new GDroite(u+d.u,v+d.v,h+d.h);
		     tableau[1] = new GDroite(u-d.u,v-d.v,h-d.h);
		     return tableau;
		}
		
		public GPoint projection(GPoint m) {
		     double t = u*m.x+v*m.y+h;
		     return new GPoint(m.x-t*u,m.y-t*v);
		}    
		
		public Geom symetrie(GPoint m) {
		     return new GDroite(u,v,-h+2*u*m.x+2*v*m.y);
		}
		
		public Geom symetrie(GDroite d) {
		     double lambda = u*d.u+v*d.v;
		     return new GDroite(u+lambda*d.u,v+lambda*d.v,h+lambda*d.h);
		}
		
		public Geom symetrie(GCercle c) {
		     double den = u*c.x+v*c.y+h;
		     double a1 = (- v*c.y*c.x-h*c.x-u*c.x*c.x+u*c.r/2)/ den;
		     double b1 = (- h*c.y-v*c.y*c.y-u*c.x*c.y+v*c.r/2)/ den;
		     double c1 = (v*c.y*c.y*c.y+u*c.x*c.y*c.y+u*c.x*c.x*c.x -u*c.r*c.x+h*c.y*c.y+v*c.y*c.x*c.x+h*c.x*c.x-v*c.r*c.y)/ den;
             return new GCercle(-a1,-b1,Math.sqrt(a1*a1+b1*b1-c1));
        }       
        
        public static boolean concourantes(GDroite d1,GDroite d2,GDroite d3) {
             return (new GPoint(d1,d2)).appartient(d3);
        } 
}

public class GCercle extends Geom {
       double x,y,r;
       
       GCercle(double x,double y,double r) {
            this.x=x;
            this.y=y;
            this.r=r;
       }
       
       GCercle(GPoint m1, GPoint m2) {
            this.r = Math.sqrt(Geom.sqr(m2.x-m1.x)+Geom.sqr(m2.y-m1.y));
            this.x = m1.x;
            this.y = m1.y;
       }
       
       GCercle(GPoint m, double r) {
            this.r = r;
            this.x = m.x;
            this.y = m.y;
       }
       
       public GPoint centre() {
            return new GPoint(x,y);
       }
       
       public double rayon() {
            return r;
       }
       
       public GDroite axeRadical(GCercle c) {
            return new GDroite(x-c.x,y-c.y,
                               Geom.sqr(c.r)-Geom.sqr(r)+Geom.sqr(x)-Geom.sqr(c.x)
                               +Geom.sqr(y)-Geom.sqr(c.y));
       }
       
       public GPoint[] intersections(GDroite d) {
            return d.intersections(this);
       }
       
       public GPoint[] intersections(GCercle c) {
            return intersections(axeRadical(c));
       }
       
       public Geom symetrie(GPoint m) {
             return new GCercle(2*m.x-x,2*m.y-y,r);
       }
       
       public Geom symetrie(GDroite d) {
             double t = d.u*x+d.v*y+d.h;
		     return new GCercle(x-2*t*d.u,y-2*t*d.v,r);
	   }
	   
	   public Geom symetrie(GCercle c) {
	         double den = -c.x*c.x+2*c.x*x+2*c.y*y-x*x+r*r 
                -c.y*c.y-y*y;
             double a0 = -(-c.r*x+c.r*c.x-c.x*c.y*c.y+2*x*c.x*c.x 
              -c.x*c.x*c.x-y*y*c.x+r*r*c.x-x*x*c.x 
              +2*c.y*y*c.x)/den;
             double b0 = -(-c.y*c.y*c.y-c.r*y-y*y*c.y+c.y*c.r 
              -x*x*c.y+2*c.x*x*c.y+2*c.y*c.y*y 
              -c.x*c.x*c.y+r*r*c.y)/den;
             double c0 = - (c.x*c.x*c.x*c.x+c.y*c.y*c.y*c.y+c.r*c.r 
              +2*c.x*c.x*c.y*c.y-r*r*c.y*c.y 
              -2*c.x*x*c.y*c.y-2*c.y*c.y*c.y*y 
              +y*y*c.y*c.y-2*c.y*c.y*c.r-2*c.y*y*c.x*c.x    
              +x*x*c.y*c.y+x*x*c.x*c.x+2*c.r*x*c.x 
              -r*r*c.x*c.x+y*y*c.x*c.x-2*c.x*c.x*c.x*x 
              -2*c.x*c.x*c.r+2*c.r*c.y*y)/den;
              return new GCercle(-a0,-b0,Math.sqrt(a0*a0+b0*b0-c0));
         }
}

public class Triangle extends Geom {
        GPoint [] sommets;
        
        Triangle(GPoint m1,GPoint m2, GPoint m3) {
             sommets = new GPoint[3];
             sommets[0] = m1;
             sommets[1] = m2;
             sommets[2] = m3;
        }
        Triangle(GDroite d1, GDroite d2, GDroite d3) {
             sommets = new GPoint[3];
             sommets[0] = new GPoint(d2,d3);
             sommets[1] = new GPoint(d3,d1);
             sommets[2] = new GPoint(d1,d2);
        }
        
        public GPoint centreDeGravite() {
            return new GPoint((sommets[0].x+sommets[1].x+sommets[2].x)/3,
                              (sommets[0].y+sommets[1].y+sommets[2].y)/3);
        }
        
        public GPoint centreCercleCirconscrit() {
            return new GPoint(sommets[0].mediatrice(sommets[1]),sommets[0].mediatrice(sommets[2]));
        }
        
        public GCercle cercleCirconscrit() {
            return new GCercle(centreCercleCirconscrit(),sommets[0]);
        }
        
        public GPoint centreCercleInscrit() {
            double d0 = sommets[1].distance(sommets[2]);
            double d1 = sommets[2].distance(sommets[0]);
            double d2 = sommets[0].distance(sommets[1]);
            double d=d0+d1+d2;
            return new GPoint((d0*sommets[0].x+d1*sommets[1].x+d2*sommets[2].x)/d,
                              (d0*sommets[0].y+d1*sommets[1].y+d2*sommets[2].y)/d);
        }
        
        public GCercle cercleInscrit() {
            GPoint Omega = centreCercleInscrit();
            GDroite d = new GDroite(sommets[0],sommets[1]);
            return new GCercle(Omega,Omega.distance(d));
        }
        
        public GPoint orthocentre() {
            GDroite d0 = new GDroite(sommets[1],sommets[2]);
            GDroite d1 = new GDroite(sommets[2],sommets[0]);
            return new GPoint(sommets[0].perpendiculaire(d1),sommets[1].perpendiculaire(d0));
        }
        
        public GCercle cercle9Points() {
            GDroite d0 = new GDroite(sommets[1],sommets[2]);
            GDroite d1 = new GDroite(sommets[2],sommets[0]);
            GDroite d2 = new GDroite(sommets[0],sommets[1]);
            GPoint h0 = sommets[0].projection(d0);
            GPoint h1 = sommets[1].projection(d1);
            GPoint h2 = sommets[2].projection(d2);
            Triangle triangle = new Triangle(h0,h1,h2);
            return triangle.cercleCirconscrit();
        }
        
        public GDroite droiteEuler() {
            return new GDroite(centreDeGravite(),orthocentre());
        }
        
        public GPoint[] centresCerclesExinscrits() {
        	double d0 = sommets[1].distance(sommets[2]);
            double d1 = sommets[2].distance(sommets[0]);
            double d2 = sommets[0].distance(sommets[1]);
            double s0 = -d0+d1+d2;
            double s1 = d0 - d1 + d2;
            double s2 = d0 + d1 - d2;
            GPoint[] tableau = new GPoint[3];
            tableau[0]= new GPoint((-d0*sommets[0].x+d1*sommets[1].x+d2*sommets[2].x)/s0,
                              (-d0*sommets[0].y+d1*sommets[1].y+d2*sommets[2].y)/s0);
            tableau[1]= new GPoint((d0*sommets[0].x-d1*sommets[1].x+d2*sommets[2].x)/s1,
                              (d0*sommets[0].y-d1*sommets[1].y+d2*sommets[2].y)/s1);
            tableau[2]= new GPoint((d0*sommets[0].x+d1*sommets[1].x-d2*sommets[2].x)/s2,
                              (d0*sommets[0].y+d1*sommets[1].y-d2*sommets[2].y)/s2);
            return tableau;
       }
       
       public GCercle[] cerclesExinscrits() {
            GCercle[] tableau = new GCercle[3];
            GPoint[] centres = centresCerclesExinscrits();
            GDroite d0 = new GDroite(sommets[1],sommets[2]);
            GDroite d1 = new GDroite(sommets[2],sommets[0]);
            GDroite d2 = new GDroite(sommets[0],sommets[1]);
            GPoint h0 = centres[0].projection(d0);
            GPoint h1 = centres[1].projection(d1);
            GPoint h2 = centres[2].projection(d2);
            tableau[0] = new GCercle(centres[0],h0);
            tableau[1] = new GCercle(centres[1],h1);
            tableau[2] = new GCercle(centres[2],h2);
            return tableau;
        }
}

            