import java.awt.Color;
import java.io.*;

public interface Geom {
      public static final int // pour les constructions
              LIBRE = 0  ,              //  point sans contrainte 
			  LIE_DROITE = 1  ,           //  point li  une droite
			  LIE_CERCLE = 2 ,          //  point li  un cercle 
			  LIE_COURBE = 3 ,          //  point li  une courbe 
			  INTER2DROITES = 4  ,      //  point intersection de 2 droites 
			  MILIEU = 5 ,             //  milieu de 2 points 
			  CENTRE_CERCLE = 6  ,       //  centre d'un cercle donne 
			  INTER_DROITE_CERCLE1 = 7  , //  premier point d'intersection droite cercle 
			  INTER_DROITE_CERCLE2 =  8 , //  deuxime point d'intersection droite cercle 
			  INTER2CERCLES1 =  9 ,    //  premier point d'intersection cercle cercle 
			  INTER2CERCLES2 = 10  ,     //  deuxieme point d'intersection cercle cercle 
			  PROJECTION = 11  ,         //  projection d'un point sur une droite 
			  DROITE_PAR2POINTS = 13  ,   //  droite par 2 points 
			  PARALLELE = 14  ,          //  parallele menee par un point a une droite 
			  PERPENDICULAIRE = 16  ,    //  droite perpendiculaire a une droite 
			  BISSECTRICE1 = 17  ,       //  premiere bissectrice 
			  BISSECTRICE2 = 18  ,       //  deuxieme bissectrice 
			  MEDIATRICE = 19  ,         //  mediatrice de deux points 
			  CERCLE_PAR_CENTRE = 20  ,    //  cercle par centre et point 
			  CERCLE_PAR_DIAMETRE = 21  ,  //  cercle ayant un diametre donne 
			  TANGENTE = 22  ,           //  tangente  une courbe 
			  NORMALE = 23  ,            //  normale  une courbe 
			  CENTRE_DE_COURBURE = 24  ,   //  centre de Courbure  une courbe 
			  CERCLE_OSCULATEUR = 25  , //  cercle osculateur  une courbe 
			  P_SYMETRIE_POINT = 26 ,      //  symetrique d'un point par rapport a un point 
			  P_SYMETRIE_DROITE = 27  ,    //  symetrique d'un point par rapport a une droite 
			  P_SYMETRIE_CERCLE = 28 ,	   //  symetrique d'un point par rapport a un cercle 
			  D_SYMETRIE_POINT = 29 ,      //  symetrique d'une droite par rapport a un point 
			  D_SYMETRIE_DROITE = 30  ,    //  symetrique d'une droite par rapport a une droite 
			  D_SYMETRIE_CERCLE = 31 ,     //  symetrique d'une droite par rapport a un cercle 
			  C_SYMETRIE_POINT = 32 ,      //  symetrique d'un cercle par rapport a un point 
			  C_SYMETRIE_DROITE = 33  ,    //  symetrique d'une droite par rapport a un cercle 
			  C_SYMETRIE_CERCLE = 34  ,   //  symetrique d'un cercle par rapport a un cercle
			  CENTRE_GRAVITE = 35 ,   // centre de gravit d'un triangle
			  CENTRE_CERCLE_CIRCONSCRIT = 36 , // centre du cercle circonscrit d'un triangle
			  CENTRE_CERCLE_INSCRIT = 37 , // centre du cercle inscrit dans un triangle
			  AXE_RADICAL = 38 ,
			  LIEU = 39 ,
			  ENVELOPPE = 40
			  ;
			  
	    public static final int // pour les types
	          OBJET = -1, POINT = 0, DROITE = 1, CERCLE = 2, ENSEMBLE = 3, COURBE = 4;
	          
			  
		public static final double
		     PRECISION = 1e-5;   
}

public abstract class GeomObject implements Serializable {
        int type;
        String nom;
        int constr,statutConstruction;
        GeomObject pere1, pere2;
        GeomObject suivant=null,precedent=null;
        Color couleur=Color.black;
        boolean marque;
        int numero;;
        
        
        GeomObject(){
             this.constr = Geom.LIBRE;
        }
        
        GeomObject(int constr, GeomObject pere1,GeomObject pere2){
             this.constr = constr;
             this.pere1 = pere1;
             this.pere2 = pere2;
        }
        
        GeomObject(int constr, GeomObject pere1){
             this.constr = constr;
             this.pere1 = pere1;
             this.pere2 = null;
        }
        
         
        public void setNom(String nom){
             this.nom = nom;
        }
        
        public void setColor(Color c){
             couleur = c;
        }
        
        public abstract void update();
        
        
        public static double sqr(double x) {
            return x*x;
        }
        
        public static GeomObject newObjet(int constr,GeomObject pere1,GeomObject pere2){
            switch(constr){
                case Geom.LIBRE: return new GPoint(0,0);
			    case Geom.LIE_DROITE: return GPoint.newPointLieDroite(0,0,(GDroite) pere1);
			    case Geom.LIE_CERCLE: return GPoint.newPointLieCercle(0,0,(GCercle)pere1);
			    case Geom.INTER2DROITES: return GPoint.newIntersection((GDroite) pere1,(GDroite) pere2);
			    case Geom.MILIEU: return GPoint.newMilieu((GPoint) pere1,(GPoint) pere2);
			    case Geom.CENTRE_CERCLE: return GPoint.newCentreCercle((GCercle) pere1);
			    case Geom.INTER_DROITE_CERCLE1: return GPoint.newIntersection1((GDroite) pere1, (GCercle) pere2);
			    case Geom.INTER_DROITE_CERCLE2: return GPoint.newIntersection2((GDroite) pere1, (GCercle) pere2);
			    case Geom.INTER2CERCLES1: return GPoint.newIntersection1((GCercle) pere1, (GCercle) pere2);
			    case Geom.INTER2CERCLES2: return GPoint.newIntersection2((GCercle) pere1, (GCercle) pere2);
			    case Geom.PROJECTION: return GPoint.newProjection((GPoint) pere1,(GDroite) pere2);
			    case Geom.DROITE_PAR2POINTS: return GDroite.newDroitePar2Points((GPoint) pere1,(GPoint) pere2);
			    case Geom.PARALLELE: return GDroite.newParallele((GPoint) pere1,(GDroite) pere2);
			    case Geom.PERPENDICULAIRE: return GDroite.newPerpendiculaire((GPoint) pere1,(GDroite) pere2);
			    case Geom.BISSECTRICE1: return GDroite.newBissectrice1((GDroite) pere1,(GDroite) pere2);
			    case Geom.BISSECTRICE2: return GDroite.newBissectrice2((GDroite) pere1,(GDroite) pere2);
			    case Geom.MEDIATRICE: return GDroite.newMediatrice((GPoint) pere1,(GPoint) pere2);
			    case Geom.CERCLE_PAR_CENTRE: return GCercle.newCercleParCentreEtPoint((GPoint) pere1,(GPoint) pere2);
			    case Geom.CERCLE_PAR_DIAMETRE: return GCercle.newCercleParDiametre((GPoint) pere1,(GPoint) pere2);
			    case Geom.P_SYMETRIE_POINT: return GPoint.newSymetrique((GPoint) pere1,(GPoint) pere2);
			    case Geom.P_SYMETRIE_DROITE: return GPoint.newSymetrique((GPoint) pere1,(GDroite) pere2);
			    case Geom.P_SYMETRIE_CERCLE: return GPoint.newSymetrique((GPoint) pere1,(GCercle) pere2);
			    case Geom.D_SYMETRIE_POINT: return GDroite.newSymetrique((GDroite) pere1,(GPoint) pere2);
			    case Geom.D_SYMETRIE_DROITE: return GDroite.newSymetrique((GDroite) pere1,(GDroite) pere2);
			    case Geom.C_SYMETRIE_POINT: return GCercle.newSymetrique((GCercle) pere1,(GPoint) pere2); 
			    case Geom.C_SYMETRIE_DROITE: return GCercle.newSymetrique((GCercle) pere1,(GDroite) pere2);
			    case Geom.C_SYMETRIE_CERCLE: return GCercle.newSymetrique((GCercle) pere1,(GCercle) pere2);
			    case Geom.AXE_RADICAL : return GDroite.newAxeRadical((GCercle) pere1,(GCercle) pere2);
			 }
			 return null;
		}
        
        
}

public class GPoint extends GeomObject {
		double x,y;
		
		
		
		GPoint(double x,double y,int constr, GeomObject pere1,GeomObject pere2) {
		    super(constr,pere1,pere2);
			this.x=x;
			this.y=y;
			type = Geom.POINT;
			couleur=Color.black;
		}
		
		GPoint(double x,double y,int constr, GeomObject pere1) {
		    super(constr,pere1);
			this.x=x;
			this.y=y;
			type = Geom.POINT;
			couleur=Color.black;
		}
		
		GPoint(double x,double y) {
		    super(Geom.LIBRE,null,null);
			this.x=x;
			this.y=y;
			type = Geom.POINT;
			couleur=Color.black;
		}
		
		GPoint(){
		     super(Geom.LIBRE,null,null);
		     x=0;
		     y=0;
			 type = Geom.POINT;
			 couleur=Color.black;
		}
		
		GPoint(GDroite d, double t) {
		     super(Geom.LIE_DROITE,d);
		     x = -d.h*d.u-d.v*t;
		     y = -d.h*d.v+d.u*t;
			 type = Geom.POINT;
			 couleur=Color.black;
		}
		
		GPoint(GCercle c, double t) {
		     super(Geom.LIE_CERCLE,c);
		     x = c.x + c.r * Math.cos(t);
		     y = c.y + c.r * Math.sin(t);
			 type = Geom.POINT;
			 couleur=Color.black;
		}
		
	   // Calculs des points
		
		public void doIntersection(GDroite d1, GDroite d2){
			double delta = d1.u * d2.v - d1.v * d2.u;
		    x = -(d1.h * d2.v - d1.v * d2.h)/delta;
		    y = (d1.h * d2.u - d1.u * d2.h)/delta;
		    constr=Geom.INTER2DROITES;
		    pere1=d1;
		    pere2=d2;
		}
		
		public static GPoint newIntersection(GDroite d1,GDroite d2){
		    GPoint temp=new GPoint();
		    temp.doIntersection(d1,d2);
		    return temp;
		}
		
		
	    public void doMilieu(GPoint m1, GPoint m2) {
		     x=(m1.x+m2.x)/2;
		     y=(m1.y+m2.y)/2;
		     constr=Geom.MILIEU;
		     pere1=m1;
		     pere2=m2;
		}
		public static GPoint newMilieu(GPoint m1, GPoint m2){
		   GPoint temp=new GPoint();
		    temp.doMilieu(m1,m2);
		    return temp;
		}
		public void doCentreCercle(GCercle c) {
		     x=c.x;
		     y=c.y;
		     constr=Geom.CENTRE_CERCLE;
		     pere1=c;
		     pere2=null;
		}
		public static GPoint newCentreCercle(GCercle c){
		   GPoint temp=new GPoint();
		    temp.doCentreCercle(c);
		    return temp;
		}
		public void doProjection(GPoint m,GDroite d) {
		     double t = d.u*m.x+d.v*m.y+d.h;
		     x=m.x-t*d.u; y=m.y-t*d.v;
		     constr=Geom.PROJECTION;
		     pere1=m;
		     pere2=d;
		}
		public static GPoint newProjection(GPoint m,GDroite d){
		   GPoint temp=new GPoint();
		    temp.doProjection(m,d);
		    return temp;
		}
		public void doPointLieDroite(GDroite d) {
		     double t = d.u*x+d.v*y+d.h;
		     x=x-t*d.u; y=y-t*d.v;
		     constr=Geom.LIE_DROITE;
		     pere1=d;
		     pere2=null;
		}
		public static GPoint newPointLieDroite(double x, double y,GDroite d){
		   GPoint temp=new GPoint(x,y);
		    temp.doPointLieDroite(d);
		    return temp;
		}
		public void doProjection(GPoint m,GCercle c){
			double norme=Math.sqrt(GeomObject.sqr(m.x-c.x)+GeomObject.sqr(m.y-c.y));
			if(norme<Geom.PRECISION){
			    x=c.x+c.r;
			    y=c.y;
			 } else {
			    norme=c.r/norme;
			    x=c.x+(m.x-c.x)*norme;
			    y=c.y+(m.y-c.y)*norme;
			 }
		}
		public static GPoint newProjection(GPoint m,GCercle c){
		   GPoint temp=new GPoint();
		    temp.doProjection(m,c);
		    return temp;
		}
		public void doPointLieCercle(GCercle c){
			double norme=Math.sqrt(GeomObject.sqr(x-c.x)+GeomObject.sqr(y-c.y));
			if(norme<Geom.PRECISION){
			    x=c.x+c.r;
			    y=c.y;
			 } else {
			    norme=c.r/norme;
			    x=c.x+(x-c.x)*norme;
			    y=c.y+(y-c.y)*norme;
			 }
			 constr = Geom.LIE_CERCLE;
			 pere1 = c;
			 pere2 = null;
		}
		public static GPoint newPointLieCercle(double x, double y,GCercle c){
		   GPoint temp=new GPoint(x,y);
		    temp.doPointLieCercle(c);
		    return temp;
		}
		public void doSymetrique(GPoint m1, GPoint m2) {
		     x=2*m2.x-m1.x;
		     y=2*m2.y-m1.y;
		     constr=Geom.P_SYMETRIE_POINT;
		     pere1=m1;
		     pere2=m2;
		}
		
		public static GPoint newSymetrique(GPoint m1, GPoint m2){
		   GPoint temp=new GPoint();
		    temp.doSymetrique(m1,m2);
		    return temp;
		}
		
		
		public void doSymetrique(GPoint m, GDroite d) {
		     double t = d.u*m.x+d.v*m.y+d.h;
		     x=m.x-2*t*d.u;
		     y=m.y-2*t*d.v;
		     constr=Geom.P_SYMETRIE_DROITE;
		     pere1=m;
		     pere2=d;
		}
		public static GPoint newSymetrique(GPoint m, GDroite d){
		   GPoint temp=new GPoint();
		    temp.doSymetrique(m,d);
		    return temp;
		}
		
		public void doSymetrique(GPoint m, GCercle c) {
		     double lambda = c.r/(GeomObject.sqr(m.x-c.x)+GeomObject.sqr(m.y-c.y));
		     x=c.x+lambda*(m.x-c.x); 
		     y=c.y+lambda*(m.y-c.y);
		     constr=Geom.P_SYMETRIE_CERCLE;
		     pere1=m;
		     pere2=c;
		}
		public static GPoint newSymetrique(GPoint m, GCercle c){
		   GPoint temp=new GPoint();
		    temp.doSymetrique(m,c);
		    return temp;
		}
		
		public void doIntersection1(GDroite d, GCercle c) {
             double delta = Math.sqrt(GeomObject.sqr(c.r)-GeomObject.sqr(d.u*c.x+d.v*c.y+d.h));
             double b1 = -c.y*d.u*d.v+c.x*d.v*d.v-d.u*d.h;
             double b2 = -c.x*d.u*d.v+c.y*d.u*d.u-d.v*d.h;
             x=b1+d.v*delta;
             y=b2-d.u*delta;
             constr=Geom.INTER_DROITE_CERCLE1;
		     pere1=d;
		     pere2=c;
        }
        public static GPoint newIntersection1(GDroite d, GCercle c){
		   GPoint temp=new GPoint();
		    temp.doIntersection1(d,c);
		    return temp;
		}
		
        public void doIntersection2(GDroite d, GCercle c) {
             double delta = Math.sqrt(GeomObject.sqr(c.r)-GeomObject.sqr(d.u*c.x+d.v*c.y+d.h));
             double b1 = -c.y*d.u*d.v+c.x*d.v*d.v-d.u*d.h;
             double b2 = -c.x*d.u*d.v+c.y*d.u*d.u-d.v*d.h;
             x=b1-d.v*delta;
             y=b2+d.u*delta;
             constr=Geom.INTER_DROITE_CERCLE2;
		     pere1=d;
		     pere2=c;
        }
        
        public static GPoint newIntersection2(GDroite d, GCercle c){
		   GPoint temp=new GPoint();
		    temp.doIntersection2(d,c);
		    return temp;
		}
		
        public void doIntersection1(GCercle c1,GCercle c2){
             double u=2*(c2.x-c1.x),
             		v=2*(c2.y-c1.y),
             		h=GeomObject.sqr(c2.r)-GeomObject.sqr(c1.r)
             		  +GeomObject.sqr(c1.x)-GeomObject.sqr(c2.x)
                      +GeomObject.sqr(c1.y)-GeomObject.sqr(c2.y);
             double den = Math.sqrt(u*u+v*v);
             u = u/den; v=v/den; h = h/den;
             double delta = Math.sqrt(GeomObject.sqr(c2.r)-GeomObject.sqr(u*c2.x+v*c2.y+h));
             double b1 = -c2.y*u*v+c2.x*v*v-u*h;
             double b2 = -c2.x*u*v+c2.y*u*u-v*h;
             x=b1+v*delta;
             y=b2-u*delta;
             constr=Geom.INTER2CERCLES1;
		     pere1=c1;
		     pere2=c2;
        }
        
        public static GPoint newIntersection1(GCercle c1,GCercle c2){
		   GPoint temp=new GPoint();
		    temp.doIntersection1(c1,c2);
		    return temp;
		}
		
        public void doIntersection2(GCercle c1,GCercle c2){
             double u=2*(c2.x-c1.x),
             		v=2*(c2.y-c1.y),
             		h=GeomObject.sqr(c2.r)-GeomObject.sqr(c1.r)
             		  +GeomObject.sqr(c1.x)-GeomObject.sqr(c2.x)
                      +GeomObject.sqr(c1.y)-GeomObject.sqr(c2.y);
             double den = Math.sqrt(u*u+v*v);
             u = u/den; v=v/den; h = h/den;
             double delta = Math.sqrt(GeomObject.sqr(c2.r)-GeomObject.sqr(u*c2.x+v*c2.y+h));
             double b1 = -c2.y*u*v+c2.x*v*v-u*h;
             double b2 = -c2.x*u*v+c2.y*u*u-v*h;
             x=b1-v*delta;
             y=b2+u*delta;
             constr=Geom.INTER2CERCLES2;
		     pere1=c1;
		     pere2=c2;
        }
        
        public static GPoint newIntersection2(GCercle c1,GCercle c2){
		   GPoint temp=new GPoint();
		    temp.doIntersection2(c1,c2);
		    return temp;
		}
		   
        public void doCentre(GCercle c){
             x=c.x;
             y=c.y;
        }
        public static GPoint newCentre(GCercle c){
		   GPoint temp=new GPoint();
		    temp.doCentre(c);
		    return temp;
		}
		
		public void doCentreDeCourbure(GCourbe c){
		     double dx = (c.xExpr).evalueDer("t",c.t0),
		            dy = (c.yExpr).evalueDer("t",c.t0),
		            d2x = (c.xExpr).evalueDer2("t",c.t0),
		            d2y = (c.yExpr).evalueDer2("t",c.t0),facteur;
		     facteur = (dx*dx+dy*dy)/(dx*d2y-dy*d2x);
		     x=c.x-facteur*dy;
		     y=c.y+facteur*dx;
		     constr=Geom.CENTRE_DE_COURBURE;
		     pere1=c;
		     pere2=null;
		}
		
		public static GPoint newCentreDeCourbure(GCourbe c){
		   GPoint temp=new GPoint();
		    temp.doCentreDeCourbure(c);
		    return temp;
		}
		
		public void update(){
		   switch(constr){
		      case Geom.LIBRE : break;
		      case Geom.LIE_DROITE : doPointLieDroite((GDroite) pere1); break;
		      case Geom.LIE_CERCLE : doPointLieCercle((GCercle) pere1); break;
		      case Geom.INTER2DROITES : doIntersection((GDroite) pere1, (GDroite) pere2); break;
		      case Geom.MILIEU : doMilieu((GPoint) pere1, (GPoint) pere2);break;
		      case Geom.CENTRE_CERCLE : doCentreCercle((GCercle) pere1);break;
		      case Geom.INTER_DROITE_CERCLE1 : doIntersection1((GDroite) pere1, (GCercle) pere2);break;
		      case Geom.INTER_DROITE_CERCLE2 : doIntersection2((GDroite) pere1, (GCercle) pere2);break;
		      case Geom.INTER2CERCLES1 : doIntersection1((GCercle) pere1, (GCercle) pere2);break;
		      case Geom.INTER2CERCLES2 : doIntersection2((GCercle) pere1, (GCercle) pere2);break;
		      case Geom.PROJECTION : doProjection((GPoint) pere1, (GDroite) pere2);break;
		      case Geom.P_SYMETRIE_POINT : doSymetrique((GPoint) pere1,(GPoint) pere2); break;
		      case Geom.P_SYMETRIE_DROITE : doSymetrique((GPoint) pere1,(GDroite) pere2); break;
		      case Geom.CENTRE_DE_COURBURE : doCentreDeCourbure((GCourbe) pere1); break;
		    }
		 }
		      
		
		// utilitaires
		
		public double distance(GPoint m) {
		     return Math.sqrt(GeomObject.sqr(x-m.x)+GeomObject.sqr(y-m.y));
		}
		
		public double distance(GDroite d) {
		    return Math.abs(d.u*x+d.v*y+d.h);
		}
		
		public boolean proche(GPoint m, double tolerance) {
		    return ((Math.abs(x-m.x) < tolerance) && (Math.abs(y-m.y) < tolerance));
		}
		
		public boolean proche(GDroite d, double tolerance) {
		    return (Math.abs(d.u*x+d.v*y+d.h) < tolerance);
		}
		
		public boolean appartient(GDroite d) {
		    return proche(d,Geom.PRECISION);
		}
		public boolean proche(GCercle c, double tolerance) {
		    return (Math.abs(GeomObject.sqr(c.x-x)+GeomObject.sqr(c.y-y)-c.r*c.r) < tolerance);
		}
		
		public boolean appartient(GCercle c) {
		    return proche(c,Geom.PRECISION);
		}
		
		
		public static boolean alignes(GPoint m1, GPoint m2, GPoint m3) {
		    return Math.abs(m1.x*m2.y-m1.x*m3.y-m2.x*m1.y+m2.x*m3.y+m3.x*m1.y-m3.x*m2.y)<Geom.PRECISION;
		}
		
		public String toString(){
		    return "Point:x="+x+", y="+y+"\n";
		} 
}

public class GDroite extends GeomObject {
        double u,v,h;
        
        GDroite(){
            super(Geom.LIBRE,null,null);
            this.u = 0;
            this.v = 1;
            this.h = 0;
            type = Geom.DROITE;
            couleur=Color.blue;
        }
        
        GDroite(double u, double v, double h){
            super(Geom.LIBRE,null,null);
            double d = Math.sqrt(u*u+v*v);
            this.u = u/d;
            this.v = v/d;
            this.h = h/d;
            type = Geom.DROITE;
            couleur=Color.blue;
        }
        
        GDroite(double u, double v, double h, int constr, GeomObject pere1, GeomObject pere2){
            super(constr,pere1,pere2);
            double d = Math.sqrt(u*u+v*v);
            this.u = u/d;
            this.v = v/d;
            this.h = h/d;
            type = Geom.DROITE;
            couleur=Color.blue;
        }
        
        GDroite(double u, double v, double h, int constr, GeomObject pere1){
            super(constr,pere1);
            double d = Math.sqrt(u*u+v*v);
            this.u = u/d;
            this.v = v/d;
            this.h = h/d;
            type = Geom.DROITE;
            couleur=Color.blue;
        }
        
        
        GDroite(GPoint m1, GPoint m2) {
           this(m2.y-m1.y,m1.x-m2.x,m1.y*m2.x-m1.x*m2.y,Geom.DROITE_PAR2POINTS,m1,m2);
            type = Geom.DROITE;
        }
        
        
        public void normalise(){
        	double d = Math.sqrt(u*u+v*v);
            u = u/d;
            v = v/d;
            h = h/d;
        }
        
        
        // calculs des droites
        
        public void doDroitePar2Points(GPoint m1,GPoint m2){
        	u=m2.y-m1.y;
        	v=m1.x-m2.x;
        	h=m1.y*m2.x-m1.x*m2.y;
        	normalise();
        	constr=Geom.DROITE_PAR2POINTS;
		    pere1=m1;
		    pere2=m2;
        }
        public static GDroite newDroitePar2Points(GPoint m1,GPoint m2){
		   GDroite temp=new GDroite();
		    temp.doDroitePar2Points(m1,m2);
		    return temp;
		}
		
        public void doParallele(GPoint m,GDroite d){
        	u=d.u;
        	v=d.v;
        	h=-d.u*m.x-d.v*m.y;
        	constr=Geom.PARALLELE;
		    pere1=m;
		    pere2=d;
        }
        public static GDroite newParallele(GPoint m,GDroite d){
		   GDroite temp=new GDroite();
		    temp.doParallele(m,d);
		    return temp;
		}
        
        public void doParallele(GDroite d,GPoint m){
            doParallele(m,d);
        }
        public static GDroite newParallele(GDroite d,GPoint m){
		   GDroite temp=new GDroite();
		    temp.doParallele(m,d);
		    return temp;
		}
		
        public void doPerpendiculaire(GPoint m,GDroite d){
        	u=-d.v;
        	v=d.u;
        	h=d.v*m.x-d.u*m.y;
        	constr=Geom.PERPENDICULAIRE;
		    pere1=m;
		    pere2=d;
        }
        public static GDroite newPerpendiculaire(GPoint m,GDroite d){
		   GDroite temp=new GDroite();
		    temp.doPerpendiculaire(m,d);
		    return temp;
		}
        public void doPerpendiculaire(GDroite d,GPoint m){
           doPerpendiculaire(m,d);
        }
        public static GDroite newPerpendiculaire(GDroite d,GPoint m){
		   GDroite temp=new GDroite();
		    temp.doPerpendiculaire(m,d);
		    return temp;
		}
		
        public void doBissectrice1(GDroite d1,GDroite d2){
            u=d1.u+d2.u;
            v=d1.v+d2.v;
            h=d1.h+d2.h;
            normalise();
            constr=Geom.BISSECTRICE1;
		    pere1=d1;
		    pere2=d2;
        }
        public static GDroite newBissectrice1(GDroite d1,GDroite d2){
		   GDroite temp=new GDroite();
		    temp.doBissectrice1(d1,d2);
		    return temp;
		}
		
        public void doBissectrice2(GDroite d1,GDroite d2){
            u=d1.u-d2.u;
            v=d1.v-d2.v;
            h=d1.h-d2.h;
            normalise();
            constr=Geom.BISSECTRICE2;
		    pere1=d1;
		    pere2=d2;
        }
        public static GDroite newBissectrice2(GDroite d1,GDroite d2){
		   GDroite temp=new GDroite();
		    temp.doBissectrice2(d1,d2);
		    return temp;
		}
		
        public void doSymetrique(GDroite d,GPoint m){
            u=d.u;
            v=d.v;
            h=-d.h-2*d.u*m.x-2*d.v*m.y;
            constr=Geom.D_SYMETRIE_POINT;
		    pere1=d;
		    pere2=m;
        }
        public static GDroite newSymetrique(GDroite d,GPoint m){
		   GDroite temp=new GDroite();
		    temp.doSymetrique(d,m);
		    return temp;
		}
		
        public void doSymetrique(GDroite d1,GDroite d2){
        	double lambda = d1.u*d2.u+d1.v*d2.v;
		    u=d1.u-2*lambda*d2.u;
		    v=d1.v-2*lambda*d2.v;
		    h=d1.h-2*lambda*d2.h;
		    normalise();
		    constr=Geom.D_SYMETRIE_DROITE;
		    pere1=d1;
		    pere2=d2;
        }
        public static GDroite newSymetrique(GDroite d1,GDroite d2){
		   GDroite temp=new GDroite();
		    temp.doSymetrique(d1,d2);
		    return temp;
		}
		
        public void doMediatrice(GPoint m1,GPoint m2){
            u=m1.x-m2.x;
            v=m1.y-m2.y;
		    h=(-GeomObject.sqr(m1.x)+GeomObject.sqr(m2.x)
		       -GeomObject.sqr(m1.y)+GeomObject.sqr(m2.y))/2;
		    normalise();
		    constr=Geom.MEDIATRICE;
		    pere1=m1;
		    pere2=m2;
		}
		public static GDroite newMediatrice(GPoint m1,GPoint m2){
		   GDroite temp=new GDroite();
		    temp.doMediatrice(m1,m2);
		    return temp;
		}
		
		public void doAxeRadical(GCercle c1,GCercle c2){
			u=2*(c2.x-c1.x);
			v=2*(c2.y-c1.y);
			h=GeomObject.sqr(c2.r)-GeomObject.sqr(c1.r)+GeomObject.sqr(c1.x)-GeomObject.sqr(c2.x)
                               +GeomObject.sqr(c1.y)-GeomObject.sqr(c2.y);
            normalise();
            constr=Geom.AXE_RADICAL;
		    pere1=c1;
		    pere2=c2;
		}
		public static GDroite newAxeRadical(GCercle c1,GCercle c2){
		   GDroite temp=new GDroite();
		    temp.doAxeRadical(c1,c2);
		    return temp;
		}
		
		public void doTangente(GCourbe c){
		    double dx=(c.xExpr).evalueDer("t",c.t0),
		           dy=(c.yExpr).evalueDer("t",c.t0);
		    u=-dy;
		    v=dx;
		    h=dy*c.x-dx*c.y;
		    normalise();
		    constr=Geom.TANGENTE;
		    pere1=c;
		    pere2=null;
		}
		
		public static GDroite newTangente(GCourbe c){
		    GDroite temp=new GDroite();
		    temp.doTangente(c);
		    return temp;
		}
		    
		public void doNormale(GCourbe c){
		    double dx=(c.xExpr).evalueDer("t",c.t0),
		           dy=(c.yExpr).evalueDer("t",c.t0);
		    u=dx;
		    v=dy;
		    h=-dx*c.x-dy*c.y;
		    normalise();
		    constr=Geom.NORMALE;
		    pere1=c;
		    pere2=null;
		}
		
		public static GDroite newNormale(GCourbe c){
		    GDroite temp=new GDroite();
		    temp.doNormale(c);
		    return temp;
		}
		
		public void update(){
		    switch(constr){
		       case Geom.DROITE_PAR2POINTS : doDroitePar2Points((GPoint) pere1,(GPoint) pere2); break;
		       case Geom.PARALLELE : doParallele((GPoint) pere1, (GDroite) pere2); break;
		       case Geom.PERPENDICULAIRE : doPerpendiculaire((GPoint) pere1, (GDroite) pere2); break;
		       case Geom.BISSECTRICE1 : doBissectrice1((GDroite) pere1,(GDroite) pere2); break;
		       case Geom.BISSECTRICE2 : doBissectrice2((GDroite) pere1,(GDroite) pere2); break;
		       case Geom.MEDIATRICE : doMediatrice((GPoint) pere1,(GPoint) pere2); break;
		       case Geom.D_SYMETRIE_POINT : doSymetrique((GDroite) pere1, (GPoint) pere2); break;
		       case Geom.D_SYMETRIE_DROITE : doSymetrique((GDroite) pere1,(GDroite) pere2);break;
		       case Geom.AXE_RADICAL : doAxeRadical((GCercle) pere1,(GCercle) pere2);break;
		       case Geom.TANGENTE : doTangente((GCourbe) pere1); break;
		       case Geom.NORMALE : doNormale((GCourbe) pere1); break;
		    }
		}
		       
		
		public String toString(){
		    return "Droite:"+u+"x + "+v+"y + "+h+"=0\n";
		}
        // utilitaires
        
       public static boolean concourantes(GDroite d1,GDroite d2,GDroite d3) {
             return Math.abs(d1.u*d2.v*d3.h-d1.u*d2.h*d3.v-d2.u*d1.v*d3.h
                             +d2.u*d1.h*d3.v+d3.u*d1.v*d2.h-d3.u*d1.h*d2.v)<Geom.PRECISION;;
       } 
}

public class GCercle extends GeomObject {
       double x,y,r;
       
       GCercle(){
            super(Geom.LIBRE,null,null);
            this.x=0;
            this.y=0;
            this.r=1;
            type = Geom.CERCLE;
            couleur=Color.green;
       }
       
       GCercle(double x,double y,double r) {
            super(Geom.LIBRE,null,null);
            this.x=x;
            this.y=y;
            this.r=r;
            type = Geom.CERCLE;
            couleur=Color.green;
       }
       
       GCercle(double x,double y,double r,int constr, GeomObject pere1, GeomObject pere2) {
            super(constr,pere1,pere2);
            this.x=x;
            this.y=y;
            this.r=r;
            type = Geom.CERCLE;
            couleur=Color.green;
       }
       
       GCercle(double x,double y,double r,int constr, GeomObject pere1) {
            super(constr,pere1);
            this.x=x;
            this.y=y;
            this.r=r;
            type = Geom.CERCLE;
            couleur=Color.green;
       }
       
       public double rayon() {
            return r;
       }
       
       
       // calculs
         public void doCercleParCentreEtPoint(GPoint m1,GPoint m2){
         	x=m1.x;
         	y=m1.y;
         	r=Math.sqrt(GeomObject.sqr(m1.x-m2.x)+GeomObject.sqr(m1.y-m2.y));
         	constr=Geom.CERCLE_PAR_CENTRE;
		    pere1=m1;
		    pere2=m2;
         }
         public static GCercle newCercleParCentreEtPoint(GPoint m1,GPoint m2){
		    GCercle temp=new GCercle();
		    temp.doCercleParCentreEtPoint(m1,m2);
		    return temp;
		 }
		 public void doCercleParDiametre(GPoint m1,GPoint m2){
         	x=(m1.x+m2.x)/2;
         	y=(m1.y+m2.y)/2;
         	r=Math.sqrt(GeomObject.sqr(m1.x-x)+GeomObject.sqr(m1.y-y));
         	constr=Geom.CERCLE_PAR_DIAMETRE;
		    pere1=m1;
		    pere2=m2;
         }
         public static GCercle newCercleParDiametre(GPoint m1,GPoint m2){
		    GCercle temp=new GCercle();
		    temp.doCercleParDiametre(m1,m2);
		    return temp;
		 }
         public void doCercleParCentreEtRayon(GPoint m,double r){
         	x=m.x;
         	y=m.y;
         	this.r=r;
         }
         public static GCercle newCercleParCentreEtRayon(GPoint m,double r){
		    GCercle temp=new GCercle();
		    temp.doCercleParCentreEtRayon(m,r);
		    return temp;
		 }
         public void doSymetrique(GCercle c,GPoint m){
         	x=2*m.x-c.x;
         	y=2*m.y-c.y;
         	r=c.r;
         	constr=Geom.C_SYMETRIE_POINT;
		    pere1=c;
		    pere2=m;
         }
         public static GCercle newSymetrique(GCercle c,GPoint m){
		    GCercle temp=new GCercle();
		    temp.doSymetrique(c,m);
		    return temp;
		 }
         public void doSymetrique(GCercle c,GDroite d){
         	double t = d.u*c.x+d.v*c.y+d.h;
		    x=c.x-2*t*d.u;
		    y=c.y-2*t*d.v;
		    r=c.r;
		    constr=Geom.C_SYMETRIE_DROITE;
		    pere1=c;
		    pere2=d;
         }
         public static GCercle newSymetrique(GCercle c,GDroite d){
		    GCercle temp=new GCercle();
		    temp.doSymetrique(c,d);
		    return temp;
		 }
         public void doSymetrique(GDroite d,GCercle c){   
         	 double den = d.u*c.x+d.v*c.y+d.h;
		     double a1 = (- d.v*c.y*c.x-d.h*c.x-d.u*c.x*c.x+d.u*c.r/2)/ den;
		     double b1 = (- d.h*c.y-d.v*c.y*c.y-d.u*c.x*c.y+d.v*c.r/2)/ den;
		     double c1 = (d.v*c.y*c.y*c.y+d.u*c.x*c.y*c.y+d.u*c.x*c.x*c.x 
		          -d.u*c.r*c.x+d.h*c.y*c.y+d.v*c.y*c.x*c.x+d.h*c.x*c.x-d.v*c.r*c.y)/ den;
             x=-a1;
             y=-b1;
             r=Math.sqrt(a1*a1+b1*b1-c1);
             constr=Geom.D_SYMETRIE_CERCLE;
		     pere1=d;
		     pere2=c;
         }
         public static GCercle newSymetrique(GDroite d,GCercle c){
		    GCercle temp=new GCercle();
		    temp.doSymetrique(d,c);
		    return temp;
		 }
         public void doSymetrique(GCercle c1,GCercle c2){
         	 double den = -c2.x*c2.x+2*c2.x*c1.x+2*c2.y*c1.y-c1.x*c1.x+c1.r*c1.r 
                -c2.y*c2.y-c1.y*c1.y;
             double a0 = -(-c2.r*c1.x+c2.r*c2.x-c2.x*c2.y*c2.y+2*c1.x*c2.x*c2.x 
              -c2.x*c2.x*c2.x-c1.y*c1.y*c2.x+c1.r*c1.r*c2.x-c1.x*c1.x*c2.x 
              +2*c2.y*c1.y*c2.x)/den;
             double b0 = -(-c2.y*c2.y*c2.y-c2.r*c1.y-c1.y*c1.y*c2.y+c2.y*c2.r 
              -c1.x*c1.x*c2.y+2*c2.x*c1.x*c2.y+2*c2.y*c2.y*c1.y 
              -c2.x*c2.x*c2.y+c1.r*c1.r*c2.y)/den;
             double c0 = - (c2.x*c2.x*c2.x*c2.x+c2.y*c2.y*c2.y*c2.y+c2.r*c2.r 
              +2*c2.x*c2.x*c2.y*c2.y-c1.r*c1.r*c2.y*c2.y 
              -2*c2.x*c1.x*c2.y*c2.y-2*c2.y*c2.y*c2.y*c1.y 
              +c1.y*c1.y*c2.y*c2.y-2*c2.y*c2.y*c2.r-2*c2.y*c1.y*c2.x*c2.x    
              +c1.x*c1.x*c2.y*c2.y+c1.x*c1.x*c2.x*c2.x+2*c2.r*c1.x*c2.x 
              -c1.r*c1.r*c2.x*c2.x+c1.y*c1.y*c2.x*c2.x-2*c2.x*c2.x*c2.x*c1.x 
              -2*c2.x*c2.x*c2.r+2*c2.r*c2.y*c1.y)/den;
              x=-a0;
              y=-b0;
              r=Math.sqrt(a0*a0+b0*b0-c0);
              constr=Geom.C_SYMETRIE_CERCLE;
		      pere1=c1;
		      pere2=c2;
         }
         
         public static GCercle newSymetrique(GCercle c1,GCercle c2){
		    GCercle temp=new GCercle();
		    temp.doSymetrique(c1,c2);
		    return temp;
		 } 
		 
		 public void doCercleOsculateur(GCourbe c){
		     double dx = (c.xExpr).evalueDer("t",c.t0),
		            dy = (c.yExpr).evalueDer("t",c.t0),
		            d2x = (c.xExpr).evalueDer2("t",c.t0),
		            d2y = (c.yExpr).evalueDer2("t",c.t0),facteur;
		     facteur = (dx*dx+dy*dy)/(dx*d2y-dy*d2x);
		     x=c.x-facteur*dy;
		     y=c.y+facteur*dx;
		     r=facteur*Math.sqrt(dx*dx+dy*dy);
		     constr=Geom.CERCLE_OSCULATEUR;
		     pere1=c;
		     pere2=null;
		}
		
		public static GCercle newCercleOsculateur(GCourbe c){
		   GCercle temp=new GCercle();
		    temp.doCercleOsculateur(c);
		    return temp;
		}
		 
		 public void update(){
		    switch(constr){
		       case Geom.CERCLE_PAR_CENTRE : doCercleParCentreEtPoint((GPoint) pere1,(GPoint) pere2); break;
		       case Geom.CERCLE_PAR_DIAMETRE : doCercleParDiametre((GPoint) pere1, (GPoint) pere2); break;
		       case Geom.C_SYMETRIE_POINT : doSymetrique((GCercle) pere1, (GPoint) pere2); break;
		       case Geom.C_SYMETRIE_DROITE : doSymetrique((GCercle) pere1,(GDroite) pere2);break;
		       case Geom.CERCLE_OSCULATEUR : doCercleOsculateur((GCourbe) pere1); break;
		    }
		}
         
         public String toString(){
              return "Cercle: x="+x+", y="+y+", r="+r+"\n";
         }  
         
}

 
 class GCourbe extends GPoint {
       String fctX,fctY;
       double tMin,tMax,t0,delta;
       int numPoints;
       Expression xExpr,yExpr;
       double[] ptsListe;
       
       GCourbe(){
          super(0,0);
          this.fctX = "t";
          this.fctY = "t^2";
          this.xExpr = null;
          this.yExpr = null;
          this.tMin = -3;
          this.tMax = 3;
          this.numPoints = 200;
          this.delta = (tMax-tMin)/(numPoints-1);
          this.t0 = 0;
          type=Geom.COURBE;
          constr=Geom.LIE_COURBE;
          couleur=Color.magenta;
          
       }
       
       GCourbe(String fctX,String fctY, Expression xExpr, Expression yExpr, double tMin, double tMax, int numPoints, double t0){
          super(0,0);
          this.fctX = fctX;
          this.fctY = fctY;
          this.xExpr = xExpr;
          this.yExpr = yExpr;
          this.tMin = tMin;
          this.tMax = tMax;
          this.numPoints = numPoints;
          this.delta = (tMax-tMin)/(numPoints-1);
          this.t0 = t0;
          this.ptsListe=new double[2*numPoints];
          type=Geom.COURBE;
          constr=Geom.LIE_COURBE;
          couleur=Color.magenta;
          doCourbe();
       }
       
       public void doCourbe(){
          double t;
          int i;
          delta = (tMax-tMin)/(numPoints-1);
          for(i=0, t=tMin; i<numPoints; i++,t+=delta){
               ptsListe[2*i]=xExpr.evalue("t",t);
               ptsListe[2*i+1] = yExpr.evalue("t",t);
          }
          x=xExpr.evalue("t",t0);
          y=yExpr.evalue("t",t0);
       }
       
       public void update(){
       }
 }
 
 class GEnsemble extends GeomObject{
     double[] ptsListe;
     static final double dt = 1e-4, largeur = 10;
     static final int defautNb = 200;
     int numPoints=0;
     
     GEnsemble(GPoint courbe,GeomObject objet){
         super(Geom.ENSEMBLE,courbe,objet);
         type=Geom.ENSEMBLE;
     }
     
     public void recalculeTout(){
         GeomObject courant;
         courant = pere1.suivant;
         while(courant!=null){
             if(courant.type!=Geom.ENSEMBLE) {courant.update();}
             courant = courant.suivant;
         }
     }
     
     public void doLieuDroite(){
        double xOld = ((GPoint) pere1).x,
        	   yOld = ((GPoint) pere1).y,
        	   t;
        int i;
        GPoint ptDroite = (GPoint) pere1;
        GDroite d = (GDroite) ptDroite.pere1;
        GPoint ptLieu = (GPoint) pere2;
        if(numPoints != defautNb){
            numPoints = defautNb;
            ptsListe=new double[2*numPoints];
        }
        double delta = largeur/(numPoints-1);
        for(i=0, t=-largeur/2; i<numPoints; i++,t+=delta){
            ptDroite.x = -d.h*d.u-t*d.v;
            ptDroite.y = -d.h*d.v+t*d.u;
            recalculeTout();
            ptsListe[2*i] = ptLieu.x;
            ptsListe[2*i+1] = ptLieu.y;
         }
         ptDroite.x=xOld;
         ptDroite.y=yOld;
         recalculeTout();
     }
     
     public void doLieuCercle(){
        double xOld = ((GPoint) pere1).x,
        	   yOld = ((GPoint) pere1).y,
        	   t;
        int i;
        GPoint ptCercle = (GPoint) pere1;
        GCercle c = (GCercle) ptCercle.pere1;
        GPoint ptLieu = (GPoint) pere2;
        if(numPoints != defautNb){
            numPoints = defautNb;
            ptsListe=new double[2*numPoints];
        }
        double delta = 2*Math.PI/(numPoints-1);
        for(i=0, t=-Math.PI; i<numPoints; i++,t+=delta){
            ptCercle.x = c.x+c.r*Math.cos(t);
            ptCercle.y = c.y+c.r*Math.sin(t);
            recalculeTout();
            ptsListe[2*i] = ptLieu.x;
            ptsListe[2*i+1] = ptLieu.y;
         }
         ptCercle.x=xOld;
         ptCercle.y=yOld;
         recalculeTout();
     }
   
     public void doLieuCourbe(){
        double xOld = ((GCourbe) pere1).x,
        	   yOld = ((GCourbe) pere1).y,
        	   tOld = ((GCourbe) pere1).t0,
        	   tMin = ((GCourbe) pere1).tMin,
        	   t;
        int i;
        GCourbe ptCourbe = (GCourbe) pere1;
        GPoint ptLieu = (GPoint) pere2;
        if(numPoints != ptCourbe.numPoints){
            numPoints = ptCourbe.numPoints;
            ptsListe=new double[2*numPoints];
        }
        double delta = (ptCourbe.tMax-ptCourbe.tMin)/(numPoints-1);
        for(i=0, t=tMin; i<numPoints; i++,t+=delta){
            ptCourbe.x = ptCourbe.ptsListe[2*i];
            ptCourbe.y = ptCourbe.ptsListe[2*i+1];
            ptCourbe.t0=t;
            recalculeTout();
            ptsListe[2*i] = ptLieu.x;
            ptsListe[2*i+1] = ptLieu.y;
         }
         ptCourbe.x=xOld;
         ptCourbe.y=yOld;
         ptCourbe.t0=tOld;
         recalculeTout();
     }
     
     public void doLieu(){
        switch(pere1.constr){
           case Geom.LIE_DROITE : doLieuDroite(); break;
           case Geom.LIE_CERCLE : doLieuCercle(); break;
           case Geom.LIE_COURBE : doLieuCourbe(); break;
        }
     }
     
     public static GEnsemble newLieu(GPoint c, GPoint m){
        GEnsemble temp=new GEnsemble(c,m);
        temp.constr = Geom.LIEU;
        temp.doLieu();
        return temp;
     }
     
     public void doEnveloppeDroite(){
        double xOld = ((GPoint) pere1).x,
        	   yOld = ((GPoint) pere1).y,
        	   u,v,h,u1,v1,h1,denom,t,t1;
        int i;
        GPoint ptDroite = (GPoint) pere1;
        GDroite d = (GDroite) ptDroite.pere1;
        GDroite dte = (GDroite) pere2;
        if(numPoints != defautNb){
            numPoints = defautNb;
            ptsListe=new double[2*numPoints];
        }
        double delta = largeur/(numPoints-1);
        for(i=0, t=-5; i<numPoints; i++,t+=delta){
            ptDroite.x = -d.h*d.u-t*d.v;
            ptDroite.y = -d.h*d.v+t*d.u;
            recalculeTout();
            u=dte.u; v=dte.v; h=dte.h;
            t1=t+dt;
            ptDroite.x = -d.h*d.u-t1*d.v;
            ptDroite.y = -d.h*d.v+t1*d.u;
            recalculeTout();
            u1=dte.u; v1=dte.v; h1=dte.h;
            denom = u * v1 - v * u1;
            ptsListe[2*i] = -(h * v1 - v * h1)/denom;
            ptsListe[2*i+1] = (h * u1 - u * h1)/denom;
         }
         ptDroite.x=xOld;
         ptDroite.y=yOld;
         recalculeTout();
     }
     
     public void doEnveloppeCercle(){
        double xOld = ((GPoint) pere1).x,
        	   yOld = ((GPoint) pere1).y,
        	   u,v,h,u1,v1,h1,denom,t,t1;
        int i;
        GPoint ptCercle = (GPoint) pere1;
        GCercle c = (GCercle) ptCercle.pere1;
        GDroite dte = (GDroite) pere2;
        if(numPoints != defautNb){
            numPoints = defautNb;
            ptsListe=new double[2*numPoints];
        }
        double delta = 2*Math.PI/(numPoints-1);
        for(i=0, t=-Math.PI; i<numPoints; i++,t+=delta){
            ptCercle.x = c.x+c.r*Math.cos(t);
            ptCercle.y = c.y+c.r*Math.sin(t);
            recalculeTout();
            u=dte.u; v=dte.v; h=dte.h;
            t1=t+dt;
            ptCercle.x = c.x+c.r*Math.cos(t1);
            ptCercle.y = c.y+c.r*Math.sin(t1);
            recalculeTout();
            u1=dte.u; v1=dte.v; h1=dte.h;
            denom = u * v1 - v * u1;
            ptsListe[2*i] = -(h * v1 - v * h1)/denom;
            ptsListe[2*i+1] = (h * u1 - u * h1)/denom;
         }
         ptCercle.x=xOld;
         ptCercle.y=yOld;
         recalculeTout();
     }
     
     
     
     public void doEnveloppeCourbe(){
        double xOld = ((GCourbe) pere1).x,
        	   yOld = ((GCourbe) pere1).y,
        	   tOld = ((GCourbe) pere1).t0,
        	   tMin = ((GCourbe) pere1).tMin,
        	   u,v,h,u1,v1,h1,denom,t,delta;
        int i;
        GCourbe ptCourbe = (GCourbe) pere1;
        GDroite dte = (GDroite) pere2;
        if(numPoints != ptCourbe.numPoints){
            numPoints = ptCourbe.numPoints;
            ptsListe=new double[2*ptCourbe.numPoints];
        }
        delta = (ptCourbe.tMax-ptCourbe.tMin)/(numPoints-1);
        for(i=0, t=tMin; i<numPoints; i++,t+=delta){
            ptCourbe.x = ptCourbe.ptsListe[2*i];
            ptCourbe.y = ptCourbe.ptsListe[2*i+1];
            ptCourbe.t0 = t;
            recalculeTout();
            u=dte.u; v=dte.v; h=dte.h;
            ptCourbe.x = ptCourbe.xExpr.evalue("t",t+dt);
            ptCourbe.y = ptCourbe.yExpr.evalue("t",t+dt);
            ptCourbe.t0=t+dt;
            recalculeTout();
            u1=dte.u; v1=dte.v; h1=dte.h;
            denom = u * v1 - v * u1;
            ptsListe[2*i] = -(h * v1 - v * h1)/denom;
            ptsListe[2*i+1] = (h * u1 - u * h1)/denom;
         }
         ptCourbe.x=xOld;
         ptCourbe.y=yOld;
         ptCourbe.t0=tOld;
         recalculeTout();
     }
     
     public void doEnveloppe(){
        switch(pere1.constr){
           case Geom.LIE_DROITE : doEnveloppeDroite(); break;
           case Geom.LIE_CERCLE : doEnveloppeCercle(); break;
           case Geom.LIE_COURBE : doEnveloppeCourbe(); break;
        }
     }
     
     public static GEnsemble newEnveloppe(GPoint c, GDroite d){
        GEnsemble temp=new GEnsemble(c,d);
        temp.constr = Geom.ENVELOPPE;
        temp.doEnveloppe();
        return temp;
     }
     
     public void update(){
          switch(constr){
		       case Geom.LIEU : doLieu(); break;
		       case Geom.ENVELOPPE : doEnveloppe(); break;
		  }
     }
}
       

/*
public class Triangle extends GeomObject {
        GObject pere3;
        
        Triangle(GPoint m1,GPoint m2, GPoint m3) {
             pere1 = m1;
             pere2 = m2;
             pere3 = m3;
        }
        Triangle(GDroite d1, GDroite d2, GDroite d3) {
             pere1 = new GPoint(d2,d3);
             pere2 = new GPoint(d3,d1);
             pere3 = new GPoint(d1,d2);
        }
        
        public GPoint new_centreDeGravite() {
            return new GPoint((pere1.x+pere2.x+pere3.x)/3,
                              (pere1.y+pere2.y+pere3.y)/3,
                              Geom.CENTRE_GRAVITE,this);
        }
        
        public GPoint new_centreCercleCirconscrit() {
            return new GPoint(new GPoint(pere1.mediatrice(pere2),pere1.mediatrice(pere3)),
                              Geom.CENTRE_CERCLE_CIRCONSCRIT,this);
        }
        
        public GCercle new_cercleCirconscrit() {
            return new GCercle(centreCercleCirconscrit(),pere1);
        }
        
        public GPoint new_centreCercleInscrit() {
            double d0 = pere2.distance(pere3);
            double d1 = pere3.distance(pere1);
            double d2 = pere1.distance(pere2);
            double d=d0+d1+d2;
            return new GPoint((d0*pere1.x+d1*pere2.x+d2*pere3.x)/d,
                              (d0*pere1.y+d1*pere2.y+d2*pere3.y)/d,
                              Geom.CENTRE_CERCLE_INSCRIT,this);
        }
        
        public GCercle new_cercleInscrit() {
            GPoint Omega = centreCercleInscrit();
            GDroite d = new GDroite(pere1,pere2);
            return new GCercle(Omega,Omega.projection(d));
        }
        
        public GPoint new_orthocentre() {
            GDroite d0 = new GDroite(pere2,pere3);
            GDroite d1 = new GDroite(pere3,pere1);
            return new GPoint(pere1.perpendiculaire(d1),pere2.perpendiculaire(d0));
        }
        
        public GCercle new_cercle9Points() {
            GDroite d0 = new GDroite(pere2,pere3);
            GDroite d1 = new GDroite(pere3,pere1);
            GDroite d2 = new GDroite(pere1,pere2);
            GPoint h0 = pere1.projection(d0);
            GPoint h1 = pere2.projection(d1);
            GPoint h2 = pere3.projection(d2);
            Triangle triangle = new Triangle(h0,h1,h2);
            return triangle.cercleCirconscrit();
        }
        
        public GDroite droiteEuler() {
            return new GDroite(centreDeGravite(),orthocentre());
        }
        
        public GPoint[] centresCerclesExinscrits() {
        	double d0 = pere2.distance(pere3);
            double d1 = pere3.distance(pere1);
            double d2 = pere1.distance(pere2);
            double s0 = -d0+d1+d2;
            double s1 = d0 - d1 + d2;
            double s2 = d0 + d1 - d2;
            GPoint[] tableau = new GPoint[3];
            tableau[0]= new GPoint((-d0*pere1.x+d1*pere2.x+d2*pere3.x)/s0,
                              (-d0*pere1.y+d1*pere2.y+d2*pere3.y)/s0);
            tableau[1]= new GPoint((d0*pere1.x-d1*pere2.x+d2*pere3.x)/s1,
                              (d0*pere1.y-d1*pere2.y+d2*pere3.y)/s1);
            tableau[2]= new GPoint((d0*pere1.x+d1*pere2.x-d2*pere3.x)/s2,
                              (d0*pere1.y+d1*pere2.y-d2*pere3.y)/s2);
            return tableau;
       }
       
       public GCercle[] cerclesExinscrits() {
            GCercle[] tableau = new GCercle[3];
            GPoint[] centres = centresCerclesExinscrits();
            GDroite d0 = new GDroite(pere2,pere3);
            GDroite d1 = new GDroite(pere3,pere1);
            GDroite d2 = new GDroite(pere1,pere2);
            GPoint h0 = centres[0].projection(d0);
            GPoint h1 = centres[1].projection(d1);
            GPoint h2 = centres[2].projection(d2);
            tableau[0] = new GCercle(centres[0],h0);
            tableau[1] = new GCercle(centres[1],h1);
            tableau[2] = new GCercle(centres[2],h2);
            return tableau;
        }
}
*/
            