import java.awt.*;
import java.awt.event.*;

class RCCanvas extends Canvas {
	
    int		maxHor = 300;
    int		maxVer = 300;
	double xMin,xMax,yMin,yMax;
	boolean grille=true, montreInvisibles=false;
	GeomObject premier=null,dernier=null;
	Color laCouleur=Color.black;
	
	
	
	RCCanvas(double zoom) {
		super();
		xMin=-3/zoom;
		xMax=-xMin;
		yMin=-3/zoom;
		yMax=-yMin;
	}
	
	void changeGrille() {
		grille=!grille;
		repaint();
	}
	
	
	void efface() {
		repaint();
	}
	
	void retrace() {
		repaint();
	}
	
	double tolerance(){
	     return 5*(xMax-xMin)/maxHor;
	}
	
	void actualiseCoords(){
	     Rectangle r = getBounds();
   		 maxVer = r.height;
		 maxHor = r.width;
    }
	
    public void setCouleur(Color couleur){
         laCouleur=couleur;
    }
    
    public void insere(GeomObject obj){
          if(premier==null){
               premier=obj;
               dernier=obj;
          } else {
               obj.precedent=dernier;
               dernier.suivant=obj;
               dernier=obj;
          }
          obj.couleur=laCouleur;
          repaint();
    }
    
    public void insereAvecCouleur(GeomObject obj){
          if(premier==null){
               premier=obj;
               dernier=obj;
          } else {
               obj.precedent=dernier;
               dernier.suivant=obj;
               dernier=obj;
          }
          repaint();
    }
    
    public void modifierCouleur(GeomObject obj){
          obj.couleur=laCouleur;
          repaint();
    }
    
    public void changeVisibles(){
          montreInvisibles = !montreInvisibles;
          repaint();
    }
    
    public void remove(GeomObject obj){ // retire un objet de la liste
         if(obj.precedent==null && obj.suivant == null){ // c'est le seul
             premier = null; dernier = null;
         } else if(obj.precedent==null){ // c'est le premier
             premier = obj.suivant;
             premier.precedent = null;
         } else if(obj.suivant == null) { // c'est le dernier
             dernier = obj.precedent;
             dernier.suivant = null;
         } else { // objet intermdiaire
             (obj.suivant).precedent = obj.precedent;
             (obj.precedent).suivant = obj.suivant;
         }
         
    }
    
    public void detruire(GeomObject obj){ // retire un objet et toute sa descendance
        GeomObject courant=obj;
        while(courant!=null){ // met les marques  0
            courant.marque = false;
            courant=courant.suivant;
        }
        obj.marque = true;
        courant=obj.suivant;
        while(courant!=null){ // marque tous les descendants
           if(courant.pere1!=null && courant.pere1.marque)
              { courant.marque = true; }
           if(courant.pere2!=null && courant.pere2.marque)
              { courant.marque = true; }
           courant=courant.suivant;
        }
        courant = obj;
        while(courant!=null){ // dtruit tous les descendants
           if(courant.marque) { remove(courant);}
           courant = courant.suivant;
        }
        repaint();
    }
    
    public void recalculeTout(){
         GeomObject courant;
         courant = premier;
         while(courant!=null){
             courant.update();
             courant = courant.suivant;
         }
    }
             
    public void numeroteTout(){
         GeomObject courant=premier;
         int num = 0;
         while(courant!=null){
             courant.numero = num++;
             courant = courant.suivant;
         }
    }
    
	void entreParams(double zoom) {
	}
	
	// conversions coordonnes relles <-> coordonnes cran
	int xEcran(double x) {
			return (int) ((x-xMin)/(xMax-xMin)*maxHor);
	}

    int yEcran(double y) {
		return (int) ((yMax-y)/(yMax-yMin)*maxVer);
	}
	
	double xFloat(int x) {
		return xMin+ x *(xMax-xMin)/maxHor;
	}

	double yFloat(int y) {
		return yMax- y *(yMax-yMin)/maxVer;
	}
	
	
	// Les dessins
	
	private void dessine(Graphics g,GPoint m) {
		    int x=xEcran(m.x),y=yEcran(m.y);
		    if(m.couleur==Color.white){ 
		        if(montreInvisibles)
		            g.setColor(Color.lightGray);
		        else return;
		    } else {
		        g.setColor(m.couleur);
		    }
		    g.fillOval(x-2,y-2,5,5);
	}
	
	private void dessine(Graphics g,GCourbe m) {
	        double[] coords = m.ptsListe;
	        int x1=xEcran(coords[0]),y1=yEcran(coords[1]),x2,y2;
	        if(m.couleur==Color.white){ 
		        if(montreInvisibles)
		            g.setColor(Color.lightGray);
		        else return;
		    } else {
		        g.setColor(m.couleur);
		    }
	        for(int i=1;i<m.numPoints;i++){
	            // System.out.println("x="+x1+", y="+y1);
	            x2=xEcran(coords[2*i]);
	            y2=yEcran(coords[2*i+1]);
	            g.drawLine(x1,y1,x2,y2);
	            x1=x2; y1=y2;
	        }
	        int x=xEcran(m.x),y=yEcran(m.y);
		    g.fillOval(x-2,y-2,5,5);
    }
    
    private void dessine(Graphics g,GEnsemble m) {
	        double[] coords = m.ptsListe;
	        int x1=xEcran(coords[0]),y1=yEcran(coords[1]),x2,y2;
	        if(m.couleur==Color.white){ 
		        if(montreInvisibles)
		            g.setColor(Color.lightGray);
		        else return;
		    } else {
		        g.setColor(m.couleur);
		    }
	        for(int i=1;i<m.numPoints;i++){
	            // System.out.println("x="+x1+", y="+y1);
	            x2=xEcran(coords[2*i]);
	            y2=yEcran(coords[2*i+1]);
	            g.drawLine(x1,y1,x2,y2);
	            x1=x2; y1=y2;
	        }
    }
	
	private void dessine(Graphics g,GDroite d) {
		    int x0,y0,x1,y1;
			if(Math.abs(d.v)>Math.abs(d.u)){
			   x0=0; y0=yEcran(-(d.u*xMin+d.h)/d.v);
			   x1=maxHor; y1=yEcran(-(d.u*xMax+d.h)/d.v);
			} else {
			   y0=0; x0=xEcran(-(d.v*yMax+d.h)/d.u);
			   y1=maxVer; x1=xEcran(-(d.v*yMin+d.h)/d.u);
			}
			if(d.couleur==Color.white){ 
		        if(montreInvisibles)
		            g.setColor(Color.lightGray);
		        else return;
		    } else {
		        g.setColor(d.couleur);
		    }
			g.drawLine(x0,y0,x1,y1);
	}
		
	private void dessine(Graphics g,GCercle c) {
		    int x=xEcran(c.x), y=yEcran(c.y), r=xEcran(c.x+c.r)-x;
		    if(c.couleur==Color.white){ 
		        if(montreInvisibles)
		            g.setColor(Color.lightGray);
		        else return;
		    } else {
		        g.setColor(c.couleur);
		    }
		    g.drawOval(x-r,y-r,2*r,2*r);
	}
    
    public void paint(Graphics g){
		int x1,y1,x2,y2;
		
		actualiseCoords();
		// On trace les axes
		/* g.setColor(Color.blue);
		x1=xEcran(xMin);
		x2=xEcran(xMax);
		y1=yEcran(0);
		g.drawLine(x1,y1,x2,y1);
		x1=xEcran(0);
		y1=yEcran(yMin);
		y2=yEcran(yMax);
		g.drawLine(x1,y1,x1,y2);
		
		// on trace la grille
		if(grille) {
			g.setColor(Color.green);
			for(int i=(int)Math.ceil(xMin);i<=(int)Math.floor(xMax);i++) {
				if(i!=0) {
					x1=xEcran(i);
					y1=yEcran(yMin);
					y2=yEcran(yMax);
					g.drawLine(x1,y1,x1,y2);
				}
			}
			for(int i=(int)Math.ceil(yMin);i<=(int)Math.floor(yMax);i++) {
				if(i!=0) {
					x1=xEcran(xMin);
					x2=xEcran(xMax);
					y1=yEcran(i);
					g.drawLine(x1,y1,x2,y1);
				}
			}
		} */
		// g.setColor(Color.red);
		GeomObject obj = premier;
		while(obj!=null){
		    switch(obj.type){
		        case Geom.POINT : dessine(g,(GPoint) obj); break;
		        case Geom.DROITE : dessine(g,(GDroite) obj); break;
		        case Geom.CERCLE : dessine(g,(GCercle) obj); break;
		        case Geom.COURBE : dessine(g,(GCourbe) obj); break;
		        case Geom.ENSEMBLE : dessine(g,(GEnsemble) obj); break;
		    }
		    obj=obj.suivant;
		}
	}
}