import java.io.*;

public abstract class Expression implements Serializable{
    private static final double HDER=1e-8; // pour la drive premire
    private static final double HDER2=1e-4; // pour la drive seconde
    
    
    public static Expression nouvelleExpression(double x) {
        return new DoubleExpression(x);
    }
    
    public static Expression nouvelleExpression(String s) {
        return new VarExpression(s);   
    }
    
    public static Expression nouvelleExpression(String fct, Expression x) {
        return new FctExpression(fct,x);   
    }
    
    public static Expression nouvelleExpression(String op, Expression g,Expression d) {
        return new OpExpression(op,g,d);  
    }
    
    // paramtre: valeur de la variable (le nom est ts)
    public abstract double evalue(String ts, double t);
    
    public double evalueDer(String ts, double t) {
        return (evalue(ts,t+HDER)-evalue(ts,t))/HDER;
    }
		   
    public  double evalueDer2(String ts, double t) {
        return (evalue(ts,t+HDER2)+evalue(ts,t-HDER2)-2*evalue(ts,t))/(HDER2*HDER2);
    }
    
		   
    // premier paramtre: valeur d'une variable dont le nom est ts
    // deuxime paramtre: valeur de la variable dont le nom est as
    public abstract double evalue(String ts,double t,String as,double a) ;
    
    public double evalueDer(String ts,double t,String as,double a) {
        return (evalue(ts,t+HDER,as,a)-evalue(ts,t,as,a))/HDER;
    }
		   
    public double evalueDer2(String ts,double t,String as,double a) {
        return (evalue(ts,t+HDER2,as,a)+evalue(ts,t-HDER2,as,a)-2*evalue(ts,t,as,a))/(HDER2*HDER2);
    }
    
    // premier paramtre: valeur d'une variable dont le nom est ts
    // deuxime paramtre: valeur de la variable dont le nom est us
    // troisime paramtre : ....
    public abstract double evalue(String ts,double t,String us,double u,String vs,double v) ;
    
    public double evalueDer(String ts,double t,String us,double u,String vs,double v) {
        return (evalue(ts,t+HDER,us,u,vs,v)-evalue(ts,t,us,u,vs,v))/HDER;
    }
		   
    public double evalueDer2(String ts,double t,String us,double u,String vs,double v) {
        return (evalue(ts,t+HDER2,us,u,vs,v)+evalue(ts,t-HDER2,us,u,vs,v)-2*evalue(ts,t,us,u,vs,v))/(HDER2*HDER2);
    }
   
    // premier paramtre: un tableau des noms des variables
    // deuxime paramtre: un tableau des valeurs des variables
    public abstract double evalue(String[] nomsVariables,double[] valeursVariables);
    
    public double evalueDer(String[] nomsVariables,double[] valeursVariables,String ts) {
        int place=-1;
        double eval1,eval2,temp;
        for(int i=0; i<nomsVariables.length;i++) 
            if(ts.equals(nomsVariables[i])) {
                place=i; break;
            }
        if(place==-1) return 0;
        eval2=evalue(nomsVariables,valeursVariables);
    	temp=valeursVariables[place];
        valeursVariables[place]+=HDER;
        eval1=evalue(nomsVariables,valeursVariables);
	valeursVariables[place]=temp;
        return (eval1-eval2)/HDER;
    }
    public double evalueDer2(String[] nomsVariables,double[] valeursVariables,String ts) {
        int place=-1;
        double eval1,eval2,eval3,temp;
        for(int i=0; i<nomsVariables.length;i++) 
            if(ts.equals(nomsVariables[i])) {
                place=i; break;
            }
        if(place==-1) return 0;
	temp=valeursVariables[place];
        eval2=evalue(nomsVariables,valeursVariables);
        valeursVariables[place]-=HDER2;
        eval1=evalue(nomsVariables,valeursVariables);
	valeursVariables[place]+=2*HDER;
        eval3=evalue(nomsVariables,valeursVariables);
	valeursVariables[place]=temp;
        return (eval1+eval3-2*eval2)/(HDER2*HDER2);
    }
    
    
    public abstract String toString();
    
}


// sous classe des expressions rduites  un rel
class DoubleExpression extends Expression {
        private double valeur = 0.0;
        
    DoubleExpression(double valeur) {
        this.valeur = valeur;
	}
	
    public double evalue(String ts, double t) {
        return valeur;
       }
		   
    public double evalue(String ts,double t,String as,double a) {
        return valeur;
    }
    
    public double evalue(String ts,double t,String us,double u,String vs,double v){
        return valeur;
    }
   
    public double evalue(String[] nomsVariables,double[] valeursVariables) {
        return valeur;
    }
    
    public String toString() {
        return Double.toString(valeur);
    }
}

// sous classe des expressions rduites  une variable
class VarExpression extends Expression {
     private String nom = "";
     
     VarExpression(String nom) {
         this.nom=nom;
     }
     
     public double evalue(String ts, double t) {
           if(nom.equals(ts)) return t;  
	   else return 0;
     }
     
     
    
    public double evalue(String ts,double t,String as,double a) { 
	  if(nom.equals(as)) return a;
	  else if(nom.equals(ts)) return t;
	  else return 0;
    }
    
    public double evalue(String ts,double t,String us,double u,String vs,double v){ 
	  if(nom.equals(us)) return u;
	  else if(nom.equals(vs)) return v;
	  else if(nom.equals(ts)) return t;
	  else return 0;
    }
     
    
    public double evalue(String[] nomsVariables,double[] valeursVariables) {
        for(int i=0;i<nomsVariables.length;i++) 
	            if(nom.equals(nomsVariables[i])) return valeursVariables[i];
	return 0;
    }
    
    public String toString() {
        return nom;
    }
}

// sous classe des expressions  f(expr)
class FctExpression extends Expression {
    private String nom ="";
    private Expression op;
    
    FctExpression(String nom,Expression op) {
        this.nom = nom;
        this.op = op;
    }
    
    public double evalue(String ts, double t) {
	return evalueFct(op.evalue(ts,t));
    }
		   
    
    public double evalue(String ts,double t,String as,double a) {
        return evalueFct(op.evalue(ts,t,as,a));
    }
    
    public double evalue(String ts,double t,String us,double u,String vs,double v){
        return evalueFct(op.evalue(ts,t,us,u,vs,v));
    }
   
    
    public double evalue(String[] nomsVariables,double[] valeursVariables) {
        return evalueFct(op.evalue(nomsVariables,valeursVariables));
    }
    
    
    private double evalueFct(double x) { 
	if(nom.equals("-")) return -x;
        else if(nom.equals("sin")) return Math.sin(x);
        else if(nom.equals("cos")) return Math.cos(x);
	else if(nom.equals("abs")) return Math.abs(x);
	else if(nom.equals("acos")) return Math.acos(x);
	else if(nom.equals("asin")) return Math.asin(x);
	else if(nom.equals("atan")) return Math.atan(x);
	else if(nom.equals("exp")) return Math.exp(x);
	else if(nom.equals("log")) return Math.log(x);
	else if(nom.equals("int")) return Math.ceil(x);
	else if(nom.equals("sqrt")) return Math.sqrt(x);
	else if(nom.equals("sqr")) return x*x;
	else if(nom.equals("tan")) return Math.tan(x);
	else if(nom.equals("ch")) {
	    double ex=Math.exp(x);
	    return (ex+1/ex)/2;
	}
	else if(nom.equals("sh")) {
	    double ex=Math.exp(x);
	    return (ex-1/ex)/2;
	}
	else if(nom.equals("th")) {
	    double ex=Math.exp(2*x);
	    return (ex-1)/(ex+1);
	}
	else if(nom.equals("ach")) return Math.log(x+Math.sqrt(x*x+1));
	else if(nom.equals("ash")) return Math.log(x+Math.sqrt(x*x-1));
	else if(nom.equals("ath")) return Math.log((1-x)/(1+x))/2;
	else return 0;
    }
    
    public String toString() {
	return nom+"("+op.toString()+")";
    }
}

// sous classe des expressions  expr1 op expr2
class OpExpression extends Expression {
    private String nom ="";
    private Expression gauche,droite;
    
    OpExpression(String nom, Expression gauche, Expression droite) {
        this.nom=nom;
        this.gauche = gauche;
        this.droite = droite;
    }
    
    public double evalue(String ts, double t) {
      	return evalueOp(gauche.evalue(ts,t),droite.evalue(ts,t));
    }
		   
    public double evalue(String ts,double t,String as,double a) {
      	return evalueOp(gauche.evalue(ts,t,as,a),droite.evalue(ts,t,as,a));
    }
    
    public double evalue(String ts,double t,String us,double u,String vs,double v){
      	return evalueOp(gauche.evalue(ts,t,us,u,vs,v),droite.evalue(ts,t,us,u,vs,v));
    }
   
    public double evalue(String[] nomsVariables,double[] valeursVariables) {
        return evalueOp(gauche.evalue(nomsVariables,valeursVariables),droite.evalue(nomsVariables,valeursVariables));
    }
	
    private double evalueOp(double x,double y) {
        if(nom.equals("+")) return x+y;
	else if(nom.equals("-")) return x-y;
	else if(nom.equals("*")) return x*y;
	else if(nom.equals("/")) return x/y;
	else if(nom.equals("^")) return Math.pow(x,y);
	else return 0;
    }
    
    public String toString() {
      	return "("+gauche.toString()+nom+droite.toString()+")";
    }
    
}

     
